@@ -21,6 +21,7 @@ __all__ = [
     'Launchpad',
     ]
 
+import errno
 import os
 try:
     from urllib.parse import urlsplit
@@ -28,7 +29,11 @@ except:
     from urlparse import urlsplit
 import warnings
 
-from httplib2 import proxy_info_from_environment
+try:
+    from httplib2 import proxy_info_from_environment
+except ImportError:
+    from httplib2 import ProxyInfo
+    proxy_info_from_environment = ProxyInfo.from_environment
 
 from lazr.restfulclient.resource import (
     CollectionWithKeyBasedLookup,
@@ -122,9 +127,9 @@ class LaunchpadOAuthAwareHttp(RestfulHttp):
     def _bad_oauth_token(self, response, content):
         """Helper method to detect an error caused by a bad OAuth token."""
         return (response.status == 401 and
-                (content.startswith("Expired token")
-                 or content.startswith("Invalid token")
-                 or content.startswith("Unknown access token")))
+                (content.startswith(b"Expired token")
+                 or content.startswith(b"Invalid token")
+                 or content.startswith(b"Unknown access token")))
 
     def _request(self, *args):
         response, content = super(
@@ -600,8 +605,11 @@ class Launchpad(ServiceRoot):
         if launchpadlib_dir[:1] == '~':
             raise ValueError("Must set $HOME or pass 'launchpadlib_dir' to "
                 "indicate location to store cached data")
-        if not os.path.exists(launchpadlib_dir):
+        try:
             os.makedirs(launchpadlib_dir, 0o700)
+        except OSError as err:
+            if err.errno != errno.EEXIST:
+                raise
         os.chmod(launchpadlib_dir, 0o700)
         # Determine the real service root.
         service_root = uris.lookup_service_root(service_root)
@@ -609,6 +617,9 @@ class Launchpad(ServiceRoot):
         scheme, host_name, path, query, fragment = urlsplit(service_root)
         service_root_dir = os.path.join(launchpadlib_dir, host_name)
         cache_path = os.path.join(service_root_dir, 'cache')
-        if not os.path.exists(cache_path):
+        try:
             os.makedirs(cache_path, 0o700)
+        except OSError as err:
+            if err.errno != errno.EEXIST:
+                raise
         return (service_root, launchpadlib_dir, cache_path, service_root_dir)
