@@ -1,4 +1,4 @@
-# Copyright 2010 Canonical Ltd.
+# Copyright 2010-2011 Canonical Ltd.
 
 # This file is part of launchpadlib.
 #
@@ -20,6 +20,14 @@ import os
 import tempfile
 import unittest
 
+from base64 import b64decode
+
+if bytes is str:
+    # Python 2
+    unicode_type = unicode
+else:
+    unicode_type = str
+
 from launchpadlib.testing.helpers import (
     fake_keyring,
     InMemoryKeyring,
@@ -58,7 +66,7 @@ class TestUnencryptedFileCredentialStore(CredentialStoreTestCase):
         credential = self.make_credential("consumer key")
         self.store.save(credential, "unique key")
         credential2 = self.store.load("unique key")
-        self.assertEquals(credential.consumer.key, credential2.consumer.key)
+        self.assertEqual(credential.consumer.key, credential2.consumer.key)
 
     def test_unique_id_doesnt_matter(self):
         # If a file contains a credential, that credential will be
@@ -66,7 +74,7 @@ class TestUnencryptedFileCredentialStore(CredentialStoreTestCase):
         credential = self.make_credential("consumer key")
         self.store.save(credential, "some key")
         credential2 = self.store.load("some other key")
-        self.assertEquals(credential.consumer.key, credential2.consumer.key)
+        self.assertEqual(credential.consumer.key, credential2.consumer.key)
 
     def test_file_only_contains_one_credential(self):
         # A credential file may contain only one credential. If you
@@ -78,7 +86,7 @@ class TestUnencryptedFileCredentialStore(CredentialStoreTestCase):
         self.store.save(credential1, "unique key 1")
         self.store.save(credential1, "unique key 2")
         loaded = self.store.load("unique key 1")
-        self.assertEquals(loaded.consumer.key, credential2.consumer.key)
+        self.assertEqual(loaded.consumer.key, credential2.consumer.key)
 
 
 class TestKeyringCredentialStore(CredentialStoreTestCase):
@@ -94,7 +102,7 @@ class TestKeyringCredentialStore(CredentialStoreTestCase):
             credential = self.make_credential("consumer key")
             self.store.save(credential, "unique key")
             credential2 = self.store.load("unique key")
-            self.assertEquals(
+            self.assertEqual(
                 credential.consumer.key, credential2.consumer.key)
 
     def test_lookup_by_unique_key(self):
@@ -108,11 +116,12 @@ class TestKeyringCredentialStore(CredentialStoreTestCase):
             self.store.save(credential2, "key 2")
 
             loaded1 = self.store.load("key 1")
-            self.assertEquals(
+            self.assertTrue(loaded1)
+            self.assertEqual(
                 credential1.consumer.key, loaded1.consumer.key)
 
             loaded2 = self.store.load("key 2")
-            self.assertEquals(
+            self.assertEqual(
                 credential2.consumer.key, loaded2.consumer.key)
 
     def test_reused_unique_id_overwrites_old_credential(self):
@@ -127,15 +136,14 @@ class TestKeyringCredentialStore(CredentialStoreTestCase):
             self.store.save(credential2, "the only key")
 
             loaded = self.store.load("the only key")
-            self.assertEquals(
+            self.assertEqual(
                 credential2.consumer.key, loaded.consumer.key)
 
     def test_bad_unique_id_returns_none(self):
         # Trying to load a credential without providing a good unique
         # ID will get you None.
         with fake_keyring(self.keyring):
-            self.assertEquals(None, self.store.load("no such key"))
-
+            self.assertIsNone(self.store.load("no such key"))
 
     def test_keyring_returns_unicode(self):
         # Kwallet is reported to sometimes return Unicode, which broke the
@@ -143,14 +151,54 @@ class TestKeyringCredentialStore(CredentialStoreTestCase):
         # handled correctly.  (See bug lp:877374)
         class UnicodeInMemoryKeyring(InMemoryKeyring):
             def get_password(self, service, username):
-                return unicode(
-                    super(UnicodeInMemoryKeyring, self).get_password(
-                        service, username))
+                password = super(UnicodeInMemoryKeyring, self).get_password(
+                    service, username)
+                if isinstance(password, unicode_type):
+                    password = password.encode('utf-8')
+                return password
 
         self.keyring = UnicodeInMemoryKeyring()
         with fake_keyring(self.keyring):
             credential = self.make_credential("consumer key")
+            self.assertTrue(credential)
+            # Shouldn't this test actually use a unicodish key?!
             self.store.save(credential, "unique key")
             credential2 = self.store.load("unique key")
-            self.assertEquals(
+            self.assertTrue(credential2)
+            self.assertEqual(
                 credential.consumer.key, credential2.consumer.key)
+            self.assertEqual(
+                credential.consumer.secret, credential2.consumer.secret)
+
+    def test_nonencoded_key_handled(self):
+        # For backwards compatibility with keys that are not base 64 encoded.
+
+        class UnencodedInMemoryKeyring(InMemoryKeyring):
+            def get_password(self, service, username):
+                pw = super(UnencodedInMemoryKeyring, self).get_password(
+                    service, username)
+                return b64decode(pw[5:])
+
+        self.keyring = UnencodedInMemoryKeyring()
+        with fake_keyring(self.keyring):
+            credential = self.make_credential("consumer key")
+            self.store.save(credential, "unique key")
+            credential2 = self.store.load("unique key")
+            self.assertEqual(
+                credential.consumer.key, credential2.consumer.key)
+            self.assertEqual(
+                credential.consumer.secret, credential2.consumer.secret)
+
+    def test_corrupted_key_handled(self):
+        # A corrupted password results in None being returned.
+
+        class CorruptedInMemoryKeyring(InMemoryKeyring):
+            def get_password(self, service, username):
+                return "bad"
+
+        self.keyring = CorruptedInMemoryKeyring()
+        with fake_keyring(self.keyring):
+            credential = self.make_credential("consumer key")
+            self.store.save(credential, "unique key")
+            credential2 = self.store.load("unique key")
+            self.assertIsNone(credential2)
